`hdpmPredictSetup` <-
function(j=1,     # study index
           r=0,     # indicator for including (0) or not (1) the common msr
           n=100,   # max MCMC iterations to use
           p=NULL,  # dimension of data records (including covariates)
           sig=1,   # residual variance
           nx=22,   # size of grid for posteriore predictive profile
           xgrid=NULL, # grid for predictive
           npa=NULL,   # number of future patients
           px=NULL,    # size of covariate vector
           X=NULL,     # (npa x px) matrix of covariates for future pat's
           work.dir=NULL,   # directory to find the working files from hdpm
           header=T)
{# computes posterior predictive for future patients

  ## 1. read in covariates for desired posterior predictive
  if(is.null(X)){
    cat("\n *** Error: need X, covariates for future patients.")
    cat("\n     specify as file name or matrix.\n")
    return(-1)
  }
  if(!is.numeric(X)){
    dta <- read.table(X,header)
    X <- as.matrix(dta)
  }

  ## 2. change working directory (if requested..)
  if(!is.null(work.dir)){
    cat("\n Changing working directory to ",work.dir,"\n")
    old.dir <- getwd()  # by default work in current working directory
    setwd(work.dir)
  }

  ## 3. check paramters
  if (!is.null(p)){
    if (p != ncol(X)){
      cat("\n *** Error: need p=ncol(X).")
      cat("   Note: X should include dummy random effects (use 0, e.g.).\n")
      return(-1)
    }
  } else {
    p <- ncol(X)
  }
  if(is.null(npa))
    npa <- nrow(X)
  if(npa != nrow(X)){
    cat("\n *** Error: need npa=nrow(X).\n")
    return(-1)
  }
  if (is.null(px)){
    cat("\n *** Error: need px, dimension of covariate vector",
        "(W/O random effects")
    cat("\n     specify px.\n")
    return(-1)
  }
    
  if(is.null(xgrid)){
    cat("\n *** Error: need xgrid, grid of time points to plot predictive.")
    cat("\n     specify as pair 'lo hi' for the interval (lo,hi).\n")
    return(-1)
  }

  out <- file("init.p",open="w")
  cat(" n ",n,"\n",
      " p ",p,"\n",
      " sig ", sig,"\n",
      " nx ",nx,"\n",
      " xgrid from ", xgrid[1]," to ", xgrid[2],"\n",
      "npa ", npa, "\n",
      "px ", px,"\n",
      "data-z \n",
      rbind(format(t(X)),"\n"),
      file=out)
  close(out)

  if (length(j) != length(r)){
    cat("\n Error: need length(j)=length(r).\n")
    return(-1)
  }
  for(i in 1:length(j))
    {
      cat("\n Computing predictive for (j,r)=(",c(j[i],r[i]),
          ") i.e., study ",j[i])
      if(r[i]==0)        cat(" with ")
      else               cat(" without ")
      cat(" common measure.\n")
      .C("predict2",
                 jpredp=as.integer(j[i]),
                 rpredp=as.integer(r[i]),
                 package="hdpm")
    }

  ## change working directory back (if changed earlier..)
  if(!is.null(work.dir)){
    cat("\n Changing working directory back to ",old.dir,"\n")
    setwd(old.dir)
  }
}

