###
### hdpm.R - Demo
###

require(hdpm)

## data files
data.dir <- system.file("data",package="hdpm")
work.dir <- paste(getwd(),"/tmp/",sep="")
                  
## data files
Y <- file.path(data.dir,"CALGB.txt")
     ## data: responses
Z <- file.path(data.dir,"CALGBz.txt")  
     ## random effects (initial values) and covariates
X <- file.path(data.dir,"CALGBz0.txt")
     ## same as Z, for future patients (for prediction)

## run MCMC -- save working files in work.dir
hdpm(Y=Y,Z=Z,px=3,q=15,cc=15,work.dir=work.dir)

## post-process MCMC output for predictive inference
## save posterior predictive summaries in work.dir
hdpmPredictSetup(X=X,px=3,
                  j=c(0,1,2,1,2),r=c(0,0,0,1,1),
                  xgrid=c(0,21),work.dir=work.dir)

## plot predictive for future patients in study 1 and 2,
for(i in c(5,7)){
  hdpmPredict(j=1,r=0,i=i)
  hdpmPredict(j=2,r=0,i=i)
}

## part of the predictive corresponding to the idiosyncratic measure
for(i in c(5,7)){
  hdpmPredict(j=1,r=1,i=i, work.dir=work.dir)
  hdpmPredict(j=2,r=1,i=i, work.dir=work.dir)
}

## post-process MCMC output for fitted mean curves
fits <- hdpmFitSetup(px=3,xgrid=c(0,21))

## plot fitted mean curves 
par(mfrow=c(2,2),ask=T)
y <- as.matrix(read.table(Y,header=T)) # read in data
exs <- c(10:17,20:27,30:37,90:97)
for(i in exs){
  hdpmFit(fits,i=i)
  idx <- which(y[,1]==i-1)
  points(y[idx,2],y[idx,3],pch=19)
}
