\name{hdpm-package}
\alias{hdpm-package}
\docType{package}
\title{
  Hierarchical mixture of Dirichlet process (HDPM) models
  for repeated measurement data
}
\description{
  Inference for a mixed effects model with hierarchical DP
  mixture prior for the random effects.
  The functions are specific to the longitudinal data model for the
  application in \cite{Mueller, Quintana and Rosner (2004)}.
}
\details{
\tabular{ll}{
Package: \tab hdpm\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2007-04-15\cr
License: \tab GNU \cr
}
The function \code{hdpm} intializes and carries out Markov chain
Monte Carlo posterior simulation.
Use the functions \code{hdpmPredictSetup} and \code{hdpmPredict} to
obtain posterior predictive summaries for patients with given
covariates under each of the studies.
Use the functions \code{hdpmFitSetup} and \code{hdpmFit} to
obtain fitted profiles for the observed patients.

The functions are specific to the longitudinal data model for the
application in \cite{Mueller, Quintana and Rosner (2004).}
See the paper for details of the model. In summary, the implemented
model is as follows.
We assume a normal non-linear regression for \eqn{y_{jik}},
the k-th observation for the i-th patient in study j.
The mean function \eqn{m(\cdot;\theta_{ji})} is indexed by a
patient-specific random effects vector \eqn{\theta_{ji}}
\deqn{
  p(y_{jik} \mid \theta_{ji},\eta) =
     N(m(t_{jik},\theta_{jik}),  \sigma^2)
   }
See equation (14) of the paper for details on the definition of
the mean function.
The random effect \eqn{\theta_{ji}} arise from a random effects model for
the j-th study, which in turn is a mixture of
a measure \eqn{F_0} that is common to all studies, and an idiosyncratic
measure \eqn{F_j} that is specific to the j-th study.
\deqn{
  \theta_{ji} \sim \epsilon F_0 + (1-\epsilon) F_j
  }
The random probability measures \eqn{F_j} in turn are given a
Dirichlet process mixture of normal prior. We assume 
\deqn{
  F_j(\theta) = \int N(\mu,S)  dG_j(\mu),~ j=0,1,\ldots,J
}
with \eqn{G_j \sim DP(G^\star(\eta),\alpha)}.
Here \eqn{\eta} are hyperparameters that index the base measure of the
DP prior. We use a normal base measure and a conjugate hyperprior
\deqn{
  G^\star(\mu) = N(m,B), \mbox{ with } m \sim N(a,A),
  \mbox{ and }
  B^{-1}\sim Wishart(c,(cC)^{-1})
}
The Wisharet prior is parametrized such that \eqn{E(B^{-1}=C^{-1})}.
Let \eqn{\delta_x} denote a point mass at x.
We complete the model with the hyperpriors
\deqn{
  S^{-1} \sim W(q,(qR)^{-1}),~
  p(\epsilon) = \pi_0\delta_0+\pi_1\delta_1+(1-\pi_0-\pi_1)
               Be(a_\epsilon,b_\epsilon) 
}
Regression on covariates in the random effects model can be achieved by
including covariates in the random effects vector \eqn{\theta_{ji}}.
See \cite{Mueller et al. (2004, section 3.3)} for details.

}
\author{
Peter Mueller

Maintainer: Peter Mueller <pm@wotan.mdacc.tmc.edu>
}
\references{
   Mueller, P., Quintana, F. and Rosner, G. (2004).
  ``Hierarchical Meta-Analysis over Related Non-parametric Bayesian
  Models.''
  Journal of the Royal Statistical Society, Series B, 66, 735--749.
\keyword{ package }
\seealso{
  The functions in this package implement the application in
  \cite{Mueller et al. (2004).}
  Alternatively, the package
  \code{\link[hdpmn:hdpmn-package]{hdpmn}}
  implements the same hierarchical mixture of Dirichlet process models
  for a multivariate normal sampling model and is useful for
  a wider class of applications.
}
\note{
  The only functions in the source code that are specific
  to the application in Mueller et al. (2004) are the definition of the
  top level sampling model in the file \code{lpost.c}
  and the functions for resampling (transition probabilities in the
  MCMC) for the random effects in the source file \code{hdpm.c}. Both
  source files are in the directory \code{src} of the package
  distribution. Users with experience in coding MCMC could appropriately
  change these functions and recompile the package for a different
  longitudinal mean function.
\examples{
}
