/* =========================================================== 

   fit.c

   =========================================================== 
compute fitted values for observed patients */

#include "math.h"
#include "stdio.h"
#include "interface.h"
#include "vector.h"
#include "matrix.h"
#include "rand.h"
#include "nrutil.h"
#include "hdpm.h"
#include <stdlib.h>

#define PA 0
/* #define CTX 1 #define GM 2 */
#define DAY 1
#define WBC 2

#define Z1 0
#define Z2 1
#define Z3 2
#define T1 3
#define T2 4
#define B0 5
#define B1 6
#define B2 7

int makegrid(double *grid, int m, double from, double to);
void init_lpost();
void  init_fit();

/* =========================================================== 
  
  		 main 
  
   =========================================================== 


/* *************************************************
   global vars
 ************************************************* */

int n,    /* n MC sample points */
  /* p,      /* dim of par vector */ 
  px,     /* dim of covariate vector */
  nx;     /* n grid points on time axis */


double 
  *xgrid,
  *E,*S,*E2,
  *x,*z,
  *zi,*ti,*bi;
 
FILE
  *tfile, *zfile, *bfile,*xfile,
  *efile, *sfile, *ctrlfile, *xgridfile;

struct Model mdl;
struct Data dta;

/********************************************/
/*         main
/********************************************/
void fit()
{
  int
    i,jj,j,k,r,pa;
  double 
    wj, sumw, m, s, lp, wj2;

  init_lpost();
  init_fit();

  /*  loop over all data points */
  printf("\n Computing fit for patients...\n");
  for(r=0;r<dta.npa;r++){
    pa = r;
    x[0] = pa;
    printf("%d \n",r);

    /* fill in covariates */
    y_assgn_x(z,dta.z[pa],mdl.p1);
    /* was: z[p-2] = x[CTX];  z[p-1] = x[GM]; */

    /* initialize vectors for point i */
    x_zero(E,nx);
    x_zero(E2,nx);

    /* loop over all MC points */
    rewind(tfile);
    rewind(zfile);
    rewind(bfile);
    wj = 1.0/(1.0*n);
    for(i=0;
	(i<n) & (fscanf(tfile,"%lf ", &ti[0]) != EOF);
	i++){
      fscanDoubleArray(tfile,&ti[1],2*dta.npa-1);
      /* note: have already read first item to check for EOF */
      fscanDoubleArray(zfile,zi,3*dta.npa);
      fscanDoubleArray(bfile,bi,2*dta.npa);

      /* compute cond moments for z2,z3,t,b */
      z[0] = zi[pa*3]; z[1] = zi[pa*3+Z2]; z[2] = zi[pa*3+Z3];
      z[3] = ti[pa*2]; z[4] = ti[pa*2+1];
      z[5] = bi[pa*2]; z[6] = bi[pa*2+1];

      /* update predicted profile */
      for(k=0;k<nx;k++){
	x[DAY] = xgrid[k];
	lp = llikl_moments(x,z,&mdl,&m,&s);
	E[k] += wj*m;
	E2[k] += wj*(s*s+m*m);
      }
    } /* for i */
    if (i<n)
      n = i; /* total # records read */
    for(k=0;k<nx;k++){
      S[k] = sqrt(E2[k]-E[k]*E[k]);
    }
    fprintf(xfile,"%5.3f \n",x[0]);
    for(jj=px;jj>0;jj--)
      fprintf(xfile,"%5.3f ",dta.z[r][mdl.p1-jj]);

    fwriteDoubleArray(efile,E,1,nx);
    fwriteDoubleArray(sfile,S,1,nx);
    if (pa==8){
      openOut("tmp");
      writeDoubleArray(xgrid,1,nx);
      writeDoubleArray(E,1,nx);
      closeOut();
    }
  } /* for pa */

  fclose(efile);
  fclose(sfile);
  fclose(xfile);
}

/* =========================================================== */
/*
/*		 initialization & finish
/*
/* =========================================================== 
/********************************************/
/*         init
/********************************************/

  void  init_fit()
{
  int
    i,j;
  double
    from,to;
/* -------------------------------------------------
   allocate memory
 ------------------------------------------------- */

/* -------------------------------------------------
   read initial values & MC points
 ------------------------------------------------- */
  trim("init.fit","init-trim.fit");
  ctrlfile = openIn("init-trim.fit");
  fscanInt(ctrlfile," n ",&n);
  /* fscanInt(ctrlfile," p ",&p); */
  fscanInt(ctrlfile," px ",&px);
  fscanDouble(ctrlfile," sig ",&mdl.sig);

  fscanInt(ctrlfile," nx ",&nx);
  xgrid = dvector(0,nx);
  fscanDouble(ctrlfile," xgrid from ",&from);
  fscanDouble(ctrlfile," to ",&to);
  makegrid(xgrid,nx,from,to);
  closeIn();
  
  xgridfile = openOut("xgrid.fit");
  fwriteDoubleArray(xgridfile,xgrid,1,nx);
  fclose(xgridfile);

  /* dta.z will be used to store the 
     covariates of the predicted cases */

/* -------------------------------------------------
   allocate memory
 ------------------------------------------------- */
  dta.z =    dmatrix(0,dta.npa,0,mdl.p1);

/* -------------------------------------------------
   read *fake* data
 ------------------------------------------------- */
  openIn("data-z.giis");
  scanDoubleMatrix(" data-z ",dta.z, dta.npa, mdl.p1);
  closeIn();

/* -------------------------------------------------
   alloc mem
 ------------------------------------------------- */

  /* allocate mem */
  z = dvector(0,mdl.p1);
  x = dvector(0,dta.d);
  ti = dvector(0,dta.npa*3);
  zi = dvector(0,dta.npa*3);
  bi = dvector(0,dta.npa*3);
  S = dvector(0,nx);
  E = dvector(0,nx);
  E2 = dvector(0,nx);

/* -------------------------------------------------
   open files 
 ------------------------------------------------- */

  ctrlfile = openIn("init.fit");
  efile = openOut("e.fit");
  sfile = openOut("s.fit");
  xfile = openOut("x.fit");

  tfile = openIn("t.giis");
  zfile = openIn("z.giis");
  bfile = openIn("b.giis");

}


/* *************************************************
   makegrid
 ************************************************* 
int makegrid(double *grid, int m, double from, double to)
{
  int j;
  double step;

  step = (to-from)/(m*1.0);
  for(j=0;j <= m;j++)
    grid[j] = from+step*j;
}
*/

/* *************************************************
   init dta and mdl
 ************************************************* */
/*******************************************
         init
 *******************************************

init_lpost()
{
  int	
   i,j,k,dummy;
  double
    prd,x;
  
 -------------------------------------------------
   read pars
 ------------------------------------------------- 
 -------------------------------------------------
   read pars
 ------------------------------------------------- 
  trim("init.giis","init-trim.giis");
  openIn("init-trim.giis");
  scanInt(" data-npatients ", &dta.npa);
  scanInt(" data-nstudies ",&dta.J);
  scanInt(" model-nb ",&mdl.nb);
  scanInt(" model-p1 ", &mdl.p1);
  scanInt(" model-model",&mdl.model);

  scanInt(" data-N ",&dta.N);
  scanInt(" data-dim ",&dta.d);

  closeIn();

}


*/
