/* =========================================================== */
/*							       */
/*  lpost.c						       */
/*							       */
/* =========================================================== */

/* hem data, log posterior
   main file 'meta.c' */

#include "math.h"
#include "stdio.h"
#include "interface.h"
#include "vector.h"
#include "matrix.h"
#include "rand.h"
#include "hdpm.h"
#include "nrutil.h"
#include "mess.h"

#define NA 999999
#define PA 0
/* #define CTX 1 #define GM 2 */
/* took these two columns out of data file for volume paper */
#define DAY 1
#define WBC 2
#define STUDY 3

#define AGC 5
#define PLT 6

int ncall=0, cncall=0;
double max_logp = -10E10;

double *yhat, *zi; /* for debugging */

/* *************************************************
/* aux functins
 ************************************************* */

void llikl_init(struct Data *dta)
{
  /* initializations for later calls to llikl functions */
  yhat = dvector(0,dta->N);
  zi = dvector(0,dta->N);
}

/* 
   ft(..,t)   computes expectation function at day t
*/

/* -------------------------------------------------
   par_rand
 ------------------------------------------------- */
void par_rand(double *z, double *z1, double *z2, double *z3,
	      double *t1, double *t2, double *b0, double *b1,
	      double *b2)
{
  *z1 = z[0];  *z2 = z[1];  *z3 = z[2];
  *t1 = z[3];  *t2 = z[4];
  *b0 = z[5];  *b1 = z[6]; 
  *b2 = z[7]; /* meaningless but harmless in model 0 */
}

/* -------------------------------------------------
   et, etat, ft
 ------------------------------------------------- */
/* computes exp(bx), bx, f(x) respectively for
   model 0 & 1.
   Note: only ft treats t<t1, t1<t<t2
*/
double et(double *y, double *z, struct Model *mdl, double day)
{
  double
    t1,t2,b0,b1,b2,e;

  t1 = z[3];  t2 = z[4];  b0 = z[5];  b1 = z[6];  b2 = z[7];

  /* if logistic model */
  if (mdl->model==0){
    e = exp(b0 + b1*(day-t2));
  }
  else if (mdl->model==1){ /* complementary log-log */
    e = exp(b0+b1*(day-t2)+b2*(day-t2)*(day-t2));
  }
  return(e);
}

double etat(double *y, double *z, struct Model *mdl, double day)
{
  double
    t1,t2,b0,b1,b2,eta;

  t1 = z[3];  t2 = z[4];  b0 = z[5];
  b1 = z[6];  b2 = z[7];
  
  /* if logistic model */
  eta = (b0+b1*(day-t2));
  if (mdl->model==1)
    eta += (day-t2)*(day-t2)*b2;
  return(eta);
}

double ft(double *y, double *z, struct Model *mdl, double day)
{
  int 
    pa, npa;
  double
    fx,r,e,zi1,zi2,zi3,t1,t2,b0,b1,b2,eta,yt2;

  zi1 = z[0]; zi2 = z[1]; zi3 = z[2];
  t1 = z[3];
  t2 = z[4];
  b0 = z[5];
  b1 = z[6];
  b2 = z[7];
  
  /* if logistic model */
  if (mdl->model==0){
    if (day < t1){
      fx = zi1;
    }
    if ((t1 <= day) & (day<t2)){
      r = (day-t1)/(t2-t1);
      e = exp(b0);
      yt2 = zi2 + zi3*e/(1.0+e);
      fx = (1.0-r)*zi1 + r*yt2;
    }
    if (day >= t2){
      e = exp(b0+b1*(day-t2));
      fx = zi2 + zi3*e/(1.0+e);
    }
  }
  else if (mdl->model==1){ /* complementary log-log */
    if (day < t1){
      fx = zi1;
    }
    if ((t1<day) & (day<t2)){
      r = (day-t1)/(t2-t1);
      e = exp(b0);
      yt2 = zi2 + zi3*(1.0-e);
      fx = (1.0-r)*zi1 + r*yt2;
    }
    if (day>t2){
      e = exp(b0+b1*(day-t2)+b2*(day-t2)*(day-t2));
      fx = zi2+zi3 - zi3*e;
    }
  }
  return(fx);
}

double et2(double *y, double *z, struct Model *mdl)
{
  return(exp(z[5]));
}

/********************************************/
/*         log likelihood
/********************************************/
double llikl(struct Model *mdl, struct Data *dta)
/* returns log posterior for piecewise logistic model
   for wbc in hematology data:
   Model:
   wbc[i,t] = f(t,gm[i],ctx[i]) + eps[i,t]

   f(t,x) = z1[i]               t < t1
            "connecting line"   t1 < t < t2
            f(t,x)              t > t2
   where
   f is a logistic regr:
   f(t,x) = z3[i]*e/(1+e)   with 
   e = exp(z2[i]+b1*ctx[i]+b2*gm[i]+(t-t2)*(b3+b4*ctx[i]+b5*gm[i]))
   t1 = b6 + b8*gm + b9*ctx
   t2 = b7 + b8*gm + b9*ctx
   
   epx[i,t] is N(0,sigma)
*/
{

  int i, curr_pa, pa;
  double 
    day, wbc, sig, logl,fx, z;
  int 
    n_patients;
  double
    fxa[50],tta[50]; /* for debugging */
  int 
    ia=0;

  /* number patients */
  n_patients = dta->npa;
  sig = mdl->sig;

  /* likelihood */
  for (logl=0.0, curr_pa = -1, i=0; i<dta->N; i++){

    /* get variables in i-th row of data */
    pa = dta->y[i][PA];
    day = dta->y[i][DAY];
    wbc = dta->y[i][WBC];
    /* check if new patient in loop */
    if (pa != curr_pa){
      if (curr_pa==mdl->pltpa){
	openOut("tmp");
	writeDoubleArray(tta,ia,1);
	writeDoubleArray(fxa,ia,1);
	closeOut();
	ia=0;
      }
      curr_pa = pa;
      if (curr_pa >= n_patients) error("Logpost","Invalid patient id",1);
    }

    /* compute expectation function */
    /* for random effects model: add random patient effects */
    fx = ft(dta->y[i],dta->z[pa],mdl,day);
    if (pa==mdl->pltpa){
      if (ia < 49){
	fxa[ia] = fx;
	tta[ia] = day;
	ia += 1;
      }
    }
    
    /* compute residuals and log likl */
    z = wbc-fx;
    yhat[i] = fx;
    zi[i] = z;
    logl += -0.5*z*z/sig;
  }

  ncall++;
  
  return(logl);
}

/* *************************************************
   llikl_i
 ************************************************* */
/* log likelihood for patient i */
double llikl_i(int patid, struct Model *mdl, struct Data *dta,
	       double *zpa)
{

  int 
    i,pa;
  double 
    day, wbc, sig, logl,fx,z;

  /* number patients */
  sig = mdl->sig;

  /* likelihood */
  for (logl=0.0, i=dta->first[patid]; dta->y[i][PA]==patid; i++){
    /* get variables in i-th row of data */
    day = dta->y[i][DAY];
    wbc = dta->y[i][WBC];
    pa = dta->y[i][PA];

    /* compute expectation function */
    fx = ft(dta->y[i],zpa,mdl,day);
    
    /* compute residuals and log likl */
    z = wbc-fx;
    yhat[i] = fx;
    zi[i] = z;
    logl += -0.5*z*z/sig;
  }

  ncall++;
  
  return(logl);
}




/********************************************/
/*         likl_moments
/********************************************/
double llikl_moments(double *y, double *zpa,  struct Model *mdl, 
		     double *m, double *s)
/* returns log likelihood for observation y, par vector theta 
   posterior for piecewise logistic model 
   for wbc in hematology data:*/
{

  int 
    i;
  double 
    day, wbc,
    sig,logl, fx, z;

  /* number patients */
  sig = mdl->sig;

  /* get variables */
  day = y[DAY];
  wbc = y[WBC];
  
  /* compute expectation function */
  /* for random effects model: add random patient effects */
  fx = ft(y,zpa,mdl,day);

  /* compute residuals and log likl */
  *m = fx;
  *s = sqrt(sig);
  z = wbc-fx;
  logl = -0.5*z*z/sig - 0.5*log(2*M_PI*sig);

  return(logl);

}




